@@ -1,5 +1,7 @@
 #!/usr/bin/python
-# Copyright: Ansible Project
+# -*- coding: utf-8 -*-
+
+# Copyright: (c) 2018, Ansible Project
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
 from __future__ import absolute_import, division, print_function
@@ -29,10 +31,11 @@ options:
   zone:
     description:
       - The DNS zone to modify
-    required: true
+      - This is a required parameter, if parameter C(hosted_zone_id) is not supplied.
   hosted_zone_id:
     description:
       - The Hosted Zone ID of the DNS zone to modify
+      - This is a required parameter, if parameter C(zone) is not supplied.
     version_added: "2.0"
   record:
     description:
@@ -70,6 +73,7 @@ options:
   overwrite:
     description:
       - Whether an existing record should be overwritten on create if values do not match
+    type: bool
   retry_interval:
     description:
       - In the case that route53 is still servicing a prior request, this module will wait and try again after this many seconds. If you have many
@@ -128,8 +132,8 @@ options:
     default: 300
     version_added: "2.1"
 author:
-  - "Bruce Pennypacker (@bpennypacker)"
-  - "Mike Buzzetti <mike.buzzetti@gmail.com>"
+- Bruce Pennypacker (@bpennypacker)
+- Mike Buzzetti (@jimbydamonk)
 extends_documentation_fragment: aws
 '''
 
@@ -171,7 +175,7 @@ set:
     record:
       description: domain name for the record set
       returned: always
-      type: string
+      type: str
       sample: new.foo.com.
     region:
       description: ""
@@ -181,17 +185,17 @@ set:
     ttl:
       description: resource record cache TTL
       returned: always
-      type: string
+      type: str
       sample: '3600'
     type:
       description: record set type
       returned: always
-      type: string
+      type: str
       sample: A
     value:
       description: value
       returned: always
-      type: string
+      type: str
       sample: 52.43.18.27
     values:
       description: values
@@ -202,12 +206,12 @@ set:
     weight:
       description: weight of the record
       returned: always
-      type: string
+      type: str
       sample: '3'
     zone:
       description: zone this record set belongs to
       returned: always
-      type: string
+      type: str
       sample: foo.bar.com.
 '''
 
@@ -383,13 +387,13 @@ class TimeoutError(Exception):
     pass
 
 
-def get_zone_by_name(conn, module, zone_name, want_private, zone_id, want_vpc_id):
+def get_zone_id_by_name(conn, module, zone_name, want_private, want_vpc_id):
     """Finds a zone by name or zone_id"""
     for zone in invoke_with_throttling_retries(conn.get_zones):
         # only save this zone id if the private status of the zone matches
         # the private_zone_in boolean specified in the params
         private_zone = module.boolean(zone.config.get('PrivateZone', False))
-        if private_zone == want_private and ((zone.name == zone_name and zone_id is None) or zone.id.replace('/hostedzone/', '') == zone_id):
+        if private_zone == want_private and zone.name == zone_name:
             if want_vpc_id:
                 # NOTE: These details aren't available in other boto methods, hence the necessary
                 # extra API call
@@ -398,12 +402,12 @@ def get_zone_by_name(conn, module, zone_name, want_private, zone_id, want_vpc_id
                 # this is to deal with this boto bug: https://github.com/boto/boto/pull/2882
                 if isinstance(zone_details['VPCs'], dict):
                     if zone_details['VPCs']['VPC']['VPCId'] == want_vpc_id:
-                        return zone
+                        return zone.id
                 else:  # Forward compatibility for when boto fixes that bug
                     if want_vpc_id in [v['VPCId'] for v in zone_details['VPCs']]:
-                        return zone
+                        return zone.id
             else:
-                return zone
+                return zone.id
     return None
 
 
@@ -457,41 +461,51 @@ def invoke_with_throttling_retries(function_ref, *argv, **kwargs):
 def main():
     argument_spec = ec2_argument_spec()
     argument_spec.update(dict(
-        state=dict(aliases=['command'], choices=['present', 'absent', 'get', 'create', 'delete'], required=True),
-        zone=dict(required=True),
-        hosted_zone_id=dict(required=False, default=None),
-        record=dict(required=True),
-        ttl=dict(required=False, type='int', default=3600),
-        type=dict(choices=['A', 'CNAME', 'MX', 'AAAA', 'TXT', 'PTR', 'SRV', 'SPF', 'CAA', 'NS', 'SOA'], required=True),
-        alias=dict(required=False, type='bool'),
-        alias_hosted_zone_id=dict(required=False),
-        alias_evaluate_target_health=dict(required=False, type='bool', default=False),
-        value=dict(required=False, type='list'),
-        overwrite=dict(required=False, type='bool'),
-        retry_interval=dict(required=False, default=500),
-        private_zone=dict(required=False, type='bool', default=False),
-        identifier=dict(required=False, default=None),
-        weight=dict(required=False, type='int'),
-        region=dict(required=False),
-        health_check=dict(required=False),
-        failover=dict(required=False, choices=['PRIMARY', 'SECONDARY']),
-        vpc_id=dict(required=False),
-        wait=dict(required=False, type='bool', default=False),
-        wait_timeout=dict(required=False, type='int', default=300),
+        state=dict(type='str', required=True, choices=['absent', 'create', 'delete', 'get', 'present'], aliases=['command']),
+        zone=dict(type='str'),
+        hosted_zone_id=dict(type='str'),
+        record=dict(type='str', required=True),
+        ttl=dict(type='int', default=3600),
+        type=dict(type='str', required=True, choices=['A', 'AAAA', 'CAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SPF', 'SRV', 'TXT']),
+        alias=dict(type='bool'),
+        alias_hosted_zone_id=dict(type='str'),
+        alias_evaluate_target_health=dict(type='bool', default=False),
+        value=dict(type='list'),
+        overwrite=dict(type='bool'),
+        retry_interval=dict(type='int', default=500),
+        private_zone=dict(type='bool', default=False),
+        identifier=dict(type='str'),
+        weight=dict(type='int'),
+        region=dict(type='str'),
+        health_check=dict(type='str'),
+        failover=dict(type='str', choices=['PRIMARY', 'SECONDARY']),
+        vpc_id=dict(type='str'),
+        wait=dict(type='bool', default=False),
+        wait_timeout=dict(type='int', default=300),
     ))
 
-    # state=present, absent, create, delete THEN value is required
-    required_if = [('state', 'present', ['value']), ('state', 'create', ['value'])]
-    required_if.extend([('state', 'absent', ['value']), ('state', 'delete', ['value'])])
-
-    # If alias is True then you must specify alias_hosted_zone as well
-    required_together = [['alias', 'alias_hosted_zone_id']]
-
-    # failover, region, and weight are mutually exclusive
-    mutually_exclusive = [('failover', 'region', 'weight')]
-
-    module = AnsibleModule(argument_spec=argument_spec, required_together=required_together, required_if=required_if,
-                           mutually_exclusive=mutually_exclusive, supports_check_mode=True)
+    module = AnsibleModule(
+        argument_spec=argument_spec,
+        supports_check_mode=True,
+        required_one_of=[['zone', 'hosted_zone_id']],
+        # If alias is True then you must specify alias_hosted_zone as well
+        required_together=[['alias', 'alias_hosted_zone_id']],
+        # state=present, absent, create, delete THEN value is required
+        required_if=(
+            ('state', 'present', ['value']),
+            ('state', 'create', ['value']),
+            ('state', 'absent', ['value']),
+            ('state', 'delete', ['value']),
+        ),
+        # failover, region and weight are mutually exclusive
+        mutually_exclusive=[('failover', 'region', 'weight')],
+        # failover, region and weight require identifier
+        required_by=dict(
+            failover=('identifier',),
+            region=('identifier',),
+            weight=('identifier',),
+        ),
+    )
 
     if not HAS_BOTO:
         module.fail_json(msg='boto required for this module')
@@ -506,7 +520,7 @@ def main():
     elif module.params['state'] == 'get':
         command_in = 'get'
 
-    zone_in = module.params.get('zone').lower()
+    zone_in = (module.params.get('zone') or '').lower()
     hosted_zone_id_in = module.params.get('hosted_zone_id')
     ttl_in = module.params.get('ttl')
     record_in = module.params.get('record').lower()
@@ -542,8 +556,6 @@ def main():
     if command_in == 'create' or command_in == 'delete':
         if alias_in and len(value_in) != 1:
             module.fail_json(msg="parameter 'value' must contain a single dns name for alias records")
-        if (weight_in is not None or region_in is not None or failover_in is not None) and identifier_in is None:
-            module.fail_json(msg="If you specify failover, region or weight you must also specify identifier")
         if (weight_in is None and region_in is None and failover_in is None) and identifier_in is not None:
             module.fail_json(msg="You have specified identifier which makes sense only if you specify one of: weight, region or failover.")
 
@@ -554,11 +566,11 @@ def main():
         module.fail_json(msg=e.error_message)
 
     # Find the named zone ID
-    zone = get_zone_by_name(conn, module, zone_in, private_zone_in, hosted_zone_id_in, vpc_id_in)
+    zone_id = hosted_zone_id_in or get_zone_id_by_name(conn, module, zone_in, private_zone_in, vpc_id_in)
 
     # Verify that the requested zone is already defined in Route53
-    if zone is None:
-        errmsg = "Zone %s does not exist in Route53" % zone_in
+    if zone_id is None:
+        errmsg = "Zone %s does not exist in Route53" % (zone_in or hosted_zone_id_in)
         module.fail_json(msg=errmsg)
 
     record = {}
@@ -581,7 +593,7 @@ def main():
     if need_to_sort_records:
         wanted_rset.resource_records = sorted(unsorted_records)
 
-    sets = invoke_with_throttling_retries(conn.get_all_rrsets, zone.id, name=record_in,
+    sets = invoke_with_throttling_retries(conn.get_all_rrsets, zone_id, name=record_in,
                                           type=type_in, identifier=identifier_in)
     sets_iter = iter(sets)
     while True:
@@ -608,15 +620,12 @@ def main():
             record['type'] = rset.type
             record['record'] = decoded_name
             record['ttl'] = rset.ttl
-            if hosted_zone_id_in:
-                record['hosted_zone_id'] = hosted_zone_id_in
             record['identifier'] = rset.identifier
             record['weight'] = rset.weight
             record['region'] = rset.region
             record['failover'] = rset.failover
             record['health_check'] = rset.health_check
-            if hosted_zone_id_in:
-                record['hosted_zone_id'] = hosted_zone_id_in
+            record['hosted_zone_id'] = zone_id
             if rset.alias_dns_name:
                 record['alias'] = True
                 record['value'] = rset.alias_dns_name
@@ -651,7 +660,7 @@ def main():
     if command_in == 'delete' and not found_record:
         module.exit_json(changed=False)
 
-    changes = ResourceRecordSets(conn, zone.id)
+    changes = ResourceRecordSets(conn, zone_id)
 
     if command_in == 'create' or command_in == 'delete':
         if command_in == 'create' and found_record:
