@@ -65,8 +65,6 @@ options:
     description:
       - Whether to disable the GPG checking of signatures of packages being
         installed. Has an effect only if state is I(present) or I(latest).
-      - This setting affects packages installed from a repository as well as
-        "local" packages installed from the filesystem or a URL.
     type: bool
     default: 'no'
 
@@ -341,10 +339,7 @@ class DnfModule(YumDnf):
         For unhandled dnf.exceptions.Error scenarios, there are certain error
         messages we want to filter in an install scenario. Do that here.
         """
-        if (
-            to_text("no package matched") in to_text(error) or
-            to_text("No match for argument:") in to_text(error)
-        ):
+        if to_text("no package matched") in to_text(error):
             return "No package {0} available.".format(spec)
 
         return error
@@ -355,10 +350,7 @@ class DnfModule(YumDnf):
         messages we want to ignore in a removal scenario as known benign
         failures. Do that here.
         """
-        if (
-            'no package matched' in to_native(error) or
-            'No match for argument:' in to_native(error)
-        ):
+        if 'no package matched' in to_native(error):
             return (False, "{0} is not installed".format(spec))
 
         # Return value is tuple of:
@@ -967,14 +959,7 @@ class DnfModule(YumDnf):
                             self.module_base.install([module])
                             self.module_base.enable([module])
                         except dnf.exceptions.MarkingErrors as e:
-                            failure_response['failures'].append(
-                                " ".join(
-                                    (
-                                        ' '.join(module),
-                                        to_native(e)
-                                    )
-                                )
-                            )
+                            failure_response['failures'].append(' '.join((module, to_native(e))))
 
                 # Install groups.
                 for group in groups:
@@ -1037,14 +1022,7 @@ class DnfModule(YumDnf):
                                 response['results'].append("Module {0} upgraded.".format(module))
                             self.module_base.upgrade([module])
                         except dnf.exceptions.MarkingErrors as e:
-                            failure_response['failures'].append(
-                                " ".join(
-                                    (
-                                        ' '.join(module),
-                                        to_native(e)
-                                    )
-                                )
-                            )
+                            failure_response['failures'].append(' '.join((module, to_native(e))))
 
                 for group in groups:
                     try:
@@ -1108,14 +1086,7 @@ class DnfModule(YumDnf):
                             self.module_base.disable([module])
                             self.module_base.reset([module])
                         except dnf.exceptions.MarkingErrors as e:
-                            failure_response['failures'].append(
-                                " ".join(
-                                    (
-                                        ' '.join(module),
-                                        to_native(e)
-                                    )
-                                )
-                            )
+                            failure_response['failures'].append(' '.join((module, to_native(e))))
 
                 for group in groups:
                     try:
@@ -1187,7 +1158,7 @@ class DnfModule(YumDnf):
             else:
                 response['changed'] = True
                 if failure_response['failures']:
-                    failure_response['msg'] = 'Failed to install some of the specified packages'
+                    failure_response['msg'] = 'Failed to install some of the specified packages',
                     self.module.fail_json(**failure_response)
                 if self.module.check_mode:
                     response['msg'] = "Check mode: No changes made, but would have if not in check mode"
@@ -1205,26 +1176,6 @@ class DnfModule(YumDnf):
                         results=[],
                     )
 
-                # Validate GPG. This is NOT done in dnf.Base (it's done in the
-                # upstream CLI subclass of dnf.Base)
-                if not self.disable_gpg_check:
-                    for package in self.base.transaction.install_set:
-                        fail = False
-                        gpgres, gpgerr = self.base._sig_check_pkg(package)
-                        if gpgres == 0:  # validated successfully
-                            continue
-                        elif gpgres == 1:  # validation failed, install cert?
-                            try:
-                                self.base._get_key_for_package(package)
-                            except dnf.exceptions.Error as e:
-                                fail = True
-                        else:  # fatal error
-                            fail = True
-
-                        if fail:
-                            msg = 'Failed to validate GPG signature for {0}'.format(package)
-                            self.module.fail_json(msg=msg)
-
                 if self.download_only:
                     for package in self.base.transaction.install_set:
                         response['results'].append("Downloaded: {0}".format(package))
@@ -1237,7 +1188,7 @@ class DnfModule(YumDnf):
                         response['results'].append("Removed: {0}".format(package))
 
                 if failure_response['failures']:
-                    failure_response['msg'] = 'Failed to install some of the specified packages'
+                    failure_response['msg'] = 'Failed to install some of the specified packages',
                     self.module.exit_json(**response)
                 self.module.exit_json(**response)
         except dnf.exceptions.DepsolveError as e:
