@@ -23,15 +23,13 @@ from units.mock.loader import DictDataLoader
 from copy import deepcopy
 import uuid
 
-from ansible.compat.tests import unittest
-from ansible.compat.tests.mock import patch, MagicMock
-from ansible.errors import AnsibleError, AnsibleParserError
+from units.compat import unittest
+from units.compat.mock import patch, MagicMock
 from ansible.executor.process.worker import WorkerProcess
 from ansible.executor.task_queue_manager import TaskQueueManager
 from ansible.executor.task_result import TaskResult
 from ansible.inventory.host import Host
 from ansible.module_utils.six.moves import queue as Queue
-from ansible.playbook.block import Block
 from ansible.playbook.handler import Handler
 from ansible.plugins.strategy import StrategyBase
 
@@ -66,9 +64,7 @@ class TestStrategyBase(unittest.TestCase):
 
         mock_tqm = MagicMock(TaskQueueManager)
         mock_tqm._final_q = mock_queue
-        mock_tqm._options = MagicMock()
-        mock_tqm._notified_handlers = {}
-        mock_tqm._listening_handlers = {}
+        mock_tqm._workers = []
         strategy_base = StrategyBase(tqm=mock_tqm)
         strategy_base.cleanup()
 
@@ -95,8 +91,6 @@ class TestStrategyBase(unittest.TestCase):
         mock_tqm = MagicMock(TaskQueueManager)
         mock_tqm._final_q = mock_queue
         mock_tqm._stats = MagicMock()
-        mock_tqm._notified_handlers = {}
-        mock_tqm._listening_handlers = {}
         mock_tqm.send_callback.return_value = None
 
         for attr in ('RUN_OK', 'RUN_ERROR', 'RUN_FAILED_HOSTS', 'RUN_UNREACHABLE_HOSTS'):
@@ -110,9 +104,7 @@ class TestStrategyBase(unittest.TestCase):
 
         mock_tqm._failed_hosts = dict()
         mock_tqm._unreachable_hosts = dict()
-        mock_tqm._options = MagicMock()
-        mock_tqm._notified_handlers = {}
-        mock_tqm._listening_handlers = {}
+        mock_tqm._workers = []
         strategy_base = StrategyBase(tqm=mock_tqm)
 
         mock_host = MagicMock()
@@ -155,30 +147,31 @@ class TestStrategyBase(unittest.TestCase):
             mock_host.has_hostkey = True
             mock_hosts.append(mock_host)
 
+        mock_hosts_names = [h.name for h in mock_hosts]
+
         mock_inventory = MagicMock()
         mock_inventory.get_hosts.return_value = mock_hosts
 
         mock_tqm = MagicMock()
         mock_tqm._final_q = mock_queue
-        mock_tqm._notified_handlers = {}
-        mock_tqm._listening_handlers = {}
         mock_tqm.get_inventory.return_value = mock_inventory
 
         mock_play = MagicMock()
         mock_play.hosts = ["host%02d" % (i + 1) for i in range(0, 5)]
 
         strategy_base = StrategyBase(tqm=mock_tqm)
+        strategy_base._hosts_cache = strategy_base._hosts_cache_all = mock_hosts_names
 
         mock_tqm._failed_hosts = []
         mock_tqm._unreachable_hosts = []
-        self.assertEqual(strategy_base.get_hosts_remaining(play=mock_play), mock_hosts)
+        self.assertEqual(strategy_base.get_hosts_remaining(play=mock_play), [h.name for h in mock_hosts])
 
         mock_tqm._failed_hosts = ["host01"]
-        self.assertEqual(strategy_base.get_hosts_remaining(play=mock_play), mock_hosts[1:])
-        self.assertEqual(strategy_base.get_failed_hosts(play=mock_play), [mock_hosts[0]])
+        self.assertEqual(strategy_base.get_hosts_remaining(play=mock_play), [h.name for h in mock_hosts[1:]])
+        self.assertEqual(strategy_base.get_failed_hosts(play=mock_play), [mock_hosts[0].name])
 
         mock_tqm._unreachable_hosts = ["host02"]
-        self.assertEqual(strategy_base.get_hosts_remaining(play=mock_play), mock_hosts[2:])
+        self.assertEqual(strategy_base.get_hosts_remaining(play=mock_play), [h.name for h in mock_hosts[2:]])
         strategy_base.cleanup()
 
     @patch.object(WorkerProcess, 'run')
@@ -195,15 +188,13 @@ class TestStrategyBase(unittest.TestCase):
         mock_host.has_hostkey = True
         mock_inventory = MagicMock()
         mock_inventory.get.return_value = mock_host
-        mock_options = MagicMock()
-        mock_options.module_path = None
 
         tqm = TaskQueueManager(
             inventory=mock_inventory,
             variable_manager=mock_var_manager,
             loader=fake_loader,
-            options=mock_options,
             passwords=None,
+            forks=3,
         )
         tqm._initialize_processes(3)
         tqm.hostvars = dict()
@@ -231,8 +222,6 @@ class TestStrategyBase(unittest.TestCase):
         mock_tqm._failed_hosts = dict()
         mock_tqm._unreachable_hosts = dict()
         mock_tqm.send_callback.return_value = None
-        mock_tqm._notified_handlers = {}
-        mock_tqm._listening_handlers = {}
 
         queue_items = []
 
@@ -269,18 +258,16 @@ class TestStrategyBase(unittest.TestCase):
         mock_task._role = None
         mock_task._parent = None
         mock_task.ignore_errors = False
+        mock_task.ignore_unreachable = False
         mock_task._uuid = uuid.uuid4()
         mock_task.loop = None
         mock_task.copy.return_value = mock_task
 
-        mock_handler_task = MagicMock(Handler)
+        mock_handler_task = Handler()
         mock_handler_task.name = 'test handler'
         mock_handler_task.action = 'foo'
         mock_handler_task._parent = None
-        mock_handler_task.get_name.return_value = "test handler"
-        mock_handler_task.has_triggered.return_value = False
         mock_handler_task._uuid = 'xxxxxxxxxxxxx'
-        mock_handler_task.copy.return_value = mock_handler_task
 
         mock_iterator = MagicMock()
         mock_iterator._play = mock_play
@@ -293,9 +280,6 @@ class TestStrategyBase(unittest.TestCase):
         mock_handler_block.always = []
         mock_play.handlers = [mock_handler_block]
 
-        mock_tqm._notified_handlers = {mock_handler_task._uuid: []}
-        mock_tqm._listening_handlers = {}
-
         mock_group = MagicMock()
         mock_group.add_host.return_value = None
 
@@ -421,8 +405,7 @@ class TestStrategyBase(unittest.TestCase):
         self.assertEqual(len(results), 1)
         self.assertEqual(strategy_base._pending_results, 0)
         self.assertNotIn('test01', strategy_base._blocked_hosts)
-        self.assertIn(mock_handler_task._uuid, strategy_base._notified_handlers)
-        self.assertIn(mock_host, strategy_base._notified_handlers[mock_handler_task._uuid])
+        self.assertTrue(mock_handler_task.is_host_notified(mock_host))
 
         # queue_items.append(('set_host_var', mock_host, mock_task, None, 'foo', 'bar'))
         # results = strategy_base._process_pending_results(iterator=mock_iterator)
@@ -468,8 +451,6 @@ class TestStrategyBase(unittest.TestCase):
 
         mock_tqm = MagicMock()
         mock_tqm._final_q = mock_queue
-        mock_tqm._notified_handlers = {}
-        mock_tqm._listening_handlers = {}
 
         strategy_base = StrategyBase(tqm=mock_tqm)
         strategy_base._loader = fake_loader
@@ -506,15 +487,14 @@ class TestStrategyBase(unittest.TestCase):
         mock_worker.side_effect = fake_run
         mock_play_context = MagicMock()
 
-        mock_handler_task = MagicMock(Handler)
+        mock_handler_task = Handler()
         mock_handler_task.action = 'foo'
-        mock_handler_task.get_name.return_value = "test handler"
-        mock_handler_task.has_triggered.return_value = False
-        mock_handler_task.listen = None
+        mock_handler_task.cached_name = False
+        mock_handler_task.name = "test handler"
+        mock_handler_task.listen = []
         mock_handler_task._role = None
         mock_handler_task._parent = None
         mock_handler_task._uuid = 'xxxxxxxxxxxxxxxx'
-        mock_handler_task.copy.return_value = mock_handler_task
 
         mock_handler = MagicMock()
         mock_handler.block = [mock_handler_task]
@@ -539,25 +519,21 @@ class TestStrategyBase(unittest.TestCase):
         mock_iterator._play = mock_play
 
         fake_loader = DictDataLoader()
-        mock_options = MagicMock()
-        mock_options.module_path = None
 
         tqm = TaskQueueManager(
             inventory=mock_inventory,
             variable_manager=mock_var_mgr,
             loader=fake_loader,
-            options=mock_options,
             passwords=None,
+            forks=5,
         )
         tqm._initialize_processes(3)
-        tqm._initialize_notified_handlers(mock_play)
         tqm.hostvars = dict()
 
         try:
             strategy_base = StrategyBase(tqm=tqm)
 
             strategy_base._inventory = mock_inventory
-            strategy_base._notified_handlers = {mock_handler_task._uuid: [mock_host]}
 
             task_result = TaskResult(mock_host.name, mock_handler_task._uuid, dict(changed=False))
             strategy_base._queued_task_cache = dict()
